@@ -11,8 +11,8 @@ import (
 
 	"github.com/apex/log"
 	"github.com/ooni/probe-cli/v3/internal/cmd/oohelper/internal"
-	"github.com/ooni/probe-cli/v3/internal/engine/runtimex"
 	"github.com/ooni/probe-cli/v3/internal/engine/netx"
+	"github.com/ooni/probe-cli/v3/internal/engine/runtimex"
 )
 
 var (
@@ -24,9 +24,21 @@ var (
 	target      = flag.String("target", "", "Target URL for the test helper")
 )
 
+func newhttpclient() *http.Client {
+	// Use a nonstandard resolver, which is enough to work around the
+	// puzzling https://github.com/ooni/probe/issues/1409 issue.
+	childResolver, err := netx.NewDNSClient(
+		netx.Config{Logger: log.Log}, "dot://8.8.8.8:853")
+	runtimex.PanicOnError(err, "netx.NewDNSClient should not fail here")
+	txp := netx.NewHTTPTransport(netx.Config{
+		BaseResolver: childResolver,
+		Logger:       log.Log,
+	})
+	return &http.Client{Transport: txp}
+}
+
 func init() {
-	txp := netx.NewHTTPTransport(netx.Config{Logger: log.Log})
-	httpClient = &http.Client{Transport: txp}
+	httpClient = newhttpclient()
 	resolver = netx.NewResolver(netx.Config{Logger: log.Log})
 }
 
