@@ -8,26 +8,23 @@ import (
 	"net/http"
 	"net/url"
 	"os"
-	"path/filepath"
 	"sync"
 
 	"github.com/ooni/probe-cli/v3/internal/engine/atomicx"
 	"github.com/ooni/probe-cli/v3/internal/engine/geolocate"
-	"github.com/ooni/probe-cli/v3/internal/engine/kvstore"
 	"github.com/ooni/probe-cli/v3/internal/engine/internal/platform"
 	"github.com/ooni/probe-cli/v3/internal/engine/internal/sessionresolver"
 	"github.com/ooni/probe-cli/v3/internal/engine/internal/tunnel"
+	"github.com/ooni/probe-cli/v3/internal/engine/kvstore"
 	"github.com/ooni/probe-cli/v3/internal/engine/model"
 	"github.com/ooni/probe-cli/v3/internal/engine/netx"
 	"github.com/ooni/probe-cli/v3/internal/engine/netx/bytecounter"
 	"github.com/ooni/probe-cli/v3/internal/engine/probeservices"
-	"github.com/ooni/probe-cli/v3/internal/engine/resources"
-	"github.com/ooni/probe-cli/v3/internal/engine/version"
+	"github.com/ooni/probe-cli/v3/internal/version"
 )
 
 // SessionConfig contains the Session config
 type SessionConfig struct {
-	AssetsDir              string
 	AvailableProbeServices []model.Service
 	KVStore                KVStore
 	Logger                 model.Logger
@@ -39,9 +36,11 @@ type SessionConfig struct {
 	TorBinary              string
 }
 
-// Session is a measurement session
+// Session is a measurement session. It contains shared information
+// required to run a measurement session, and it controls the lifecycle
+// of such resources. It is not possible to reuse a Session. You MUST
+// NOT attempt to use a Session again after Session.Close.
 type Session struct {
-	assetsDir                string
 	availableProbeServices   []model.Service
 	availableTestHelpers     map[string][]model.Service
 	byteCounter              *bytecounter.Counter
@@ -62,13 +61,39 @@ type Session struct {
 	tunnelMu                 sync.Mutex
 	tunnelName               string
 	tunnel                   tunnel.Tunnel
+
+	// closeOnce allows us to call Close just once.
+	closeOnce sync.Once
+
+	// mu provides mutual exclusion.
+	mu sync.Mutex
+
+	// testLookupLocationContext is a an optional hook for testing
+	// allowing us to mock LookupLocationContext.
+	testLookupLocationContext func(ctx context.Context) (*geolocate.Results, error)
+
+	// testMaybeLookupBackendsContext is an optional hook for testing
+	// allowing us to mock MaybeLookupBackendsContext.
+	testMaybeLookupBackendsContext func(ctx context.Context) error
+
+	// testMaybeLookupLocationContext is an optional hook for testing
+	// allowing us to mock MaybeLookupLocationContext.
+	testMaybeLookupLocationContext func(ctx context.Context) error
+
+	// testNewProbeServicesClientForCheckIn is an optional hook for testing
+	// allowing us to mock NewProbeServicesClient when calling CheckIn.
+	testNewProbeServicesClientForCheckIn func(ctx context.Context) (
+		sessionProbeServicesClientForCheckIn, error)
+}
+
+// sessionProbeServicesClientForCheckIn returns the probe services
+// client that we should be using for performing the check-in.
+type sessionProbeServicesClientForCheckIn interface {
+	CheckIn(ctx context.Context, config model.CheckInConfig) (*model.CheckInInfo, error)
 }
 
 // NewSession creates a new session or returns an error
 func NewSession(config SessionConfig) (*Session, error) {
-	if config.AssetsDir == "" {
-		return nil, errors.New("AssetsDir is empty")
-	}
 	if config.Logger == nil {
 		return nil, errors.New("Logger is empty")
 	}
@@ -90,7 +115,6 @@ func NewSession(config SessionConfig) (*Session, error) {
 		return nil, err
 	}
 	sess := &Session{
-		assetsDir:               config.AssetsDir,
 		availableProbeServices:  config.AvailableProbeServices,
 		byteCounter:             bytecounter.New(),
 		kvStore:                 config.KVStore,
@@ -107,20 +131,19 @@ func NewSession(config SessionConfig) (*Session, error) {
 		ByteCounter:  sess.byteCounter,
 		BogonIsError: true,
 		Logger:       sess.logger,
+		ProxyURL:     config.ProxyURL,
+	}
+	sess.resolver = &sessionresolver.Resolver{
+		ByteCounter: sess.byteCounter,
+		KVStore:     config.KVStore,
+		Logger:      sess.logger,
+		ProxyURL:    config.ProxyURL,
 	}
-	sess.resolver = sessionresolver.New(httpConfig)
 	httpConfig.FullResolver = sess.resolver
-	httpConfig.ProxyURL = config.ProxyURL // no need to proxy the resolver
 	sess.httpDefaultTransport = netx.NewHTTPTransport(httpConfig)
 	return sess, nil
 }
 
-// ASNDatabasePath returns the path where the ASN database path should
-// be if you have called s.FetchResourcesIdempotent.
-func (s *Session) ASNDatabasePath() string {
-	return filepath.Join(s.assetsDir, resources.ASNDatabaseName)
-}
-
 // KibiBytesReceived accounts for the KibiBytes received by the HTTP clients
 // managed by this session so far, including experiments.
 func (s *Session) KibiBytesReceived() float64 {
@@ -132,29 +155,117 @@ func (s *Session) KibiBytesSent() float64 {
 	return s.byteCounter.KibiBytesSent()
 }
 
+// CheckIn calls the check-in API. The input arguments MUST NOT
+// be nil. Before querying the API, this function will ensure
+// that the config structure does not contain any field that
+// SHOULD be initialized and is not initialized. Whenever there
+// is a field that is not initialized, we will attempt to set
+// a reasonable default value for such a field. This list describes
+// the current defaults we'll choose:
+//
+// - Platform: if empty, set to Session.Platform();
+//
+// - ProbeASN: if empty, set to Session.ProbeASNString();
+//
+// - ProbeCC: if empty, set to Session.ProbeCC();
+//
+// - RunType: if empty, set to "timed";
+//
+// - SoftwareName: if empty, set to Session.SoftwareName();
+//
+// - SoftwareVersion: if empty, set to Session.SoftwareVersion();
+//
+// - WebConnectivity.CategoryCodes: if nil, we will allocate
+// an empty array (the API does not like nil).
+//
+// Because we MAY need to know the current ASN and CC, this
+// function MAY call MaybeLookupLocationContext.
+//
+// The return value is either the check-in response or an error.
+func (s *Session) CheckIn(
+	ctx context.Context, config *model.CheckInConfig) (*model.CheckInInfo, error) {
+	if err := s.maybeLookupLocationContext(ctx); err != nil {
+		return nil, err
+	}
+	client, err := s.newProbeServicesClientForCheckIn(ctx)
+	if err != nil {
+		return nil, err
+	}
+	if config.Platform == "" {
+		config.Platform = s.Platform()
+	}
+	if config.ProbeASN == "" {
+		config.ProbeASN = s.ProbeASNString()
+	}
+	if config.ProbeCC == "" {
+		config.ProbeCC = s.ProbeCC()
+	}
+	if config.RunType == "" {
+		config.RunType = "timed" // most conservative choice
+	}
+	if config.SoftwareName == "" {
+		config.SoftwareName = s.SoftwareName()
+	}
+	if config.SoftwareVersion == "" {
+		config.SoftwareVersion = s.SoftwareVersion()
+	}
+	if config.WebConnectivity.CategoryCodes == nil {
+		config.WebConnectivity.CategoryCodes = []string{}
+	}
+	return client.CheckIn(ctx, *config)
+}
+
+// maybeLookupLocationContext is a wrapper for MaybeLookupLocationContext that calls
+// the configurable testMaybeLookupLocationContext mock, if configured, and the
+// real MaybeLookupLocationContext API otherwise.
+func (s *Session) maybeLookupLocationContext(ctx context.Context) error {
+	if s.testMaybeLookupLocationContext != nil {
+		return s.testMaybeLookupLocationContext(ctx)
+	}
+	return s.MaybeLookupLocationContext(ctx)
+}
+
+// newProbeServicesClientForCheckIn is a wrapper for NewProbeServicesClientForCheckIn
+// that calls the configurable testNewProbeServicesClientForCheckIn mock, if
+// configured, and the real NewProbeServicesClient API otherwise.
+func (s *Session) newProbeServicesClientForCheckIn(
+	ctx context.Context) (sessionProbeServicesClientForCheckIn, error) {
+	if s.testNewProbeServicesClientForCheckIn != nil {
+		return s.testNewProbeServicesClientForCheckIn(ctx)
+	}
+	client, err := s.NewProbeServicesClient(ctx)
+	if err != nil {
+		return nil, err
+	}
+	return client, nil
+}
+
 // Close ensures that we close all the idle connections that the HTTP clients
 // we are currently using may have created. It will also remove the temp dir
 // that contains data from this session. Not calling this function may likely
 // cause memory leaks in your application because of open idle connections,
 // as well as excessive usage of disk space.
 func (s *Session) Close() error {
+	s.closeOnce.Do(s.doClose)
+	return nil
+}
+
+// doClose implements Close. This function is called just once.
+func (s *Session) doClose() {
 	s.httpDefaultTransport.CloseIdleConnections()
 	s.resolver.CloseIdleConnections()
 	s.logger.Infof("%s", s.resolver.Stats())
 	if s.tunnel != nil {
 		s.tunnel.Stop()
 	}
-	return os.RemoveAll(s.tempDir)
-}
-
-// CountryDatabasePath is like ASNDatabasePath but for the country DB path.
-func (s *Session) CountryDatabasePath() string {
-	return filepath.Join(s.assetsDir, resources.CountryDatabaseName)
+	_ = os.RemoveAll(s.tempDir)
 }
 
 // GetTestHelpersByName returns the available test helpers that
 // use the specified name, or false if there's none.
 func (s *Session) GetTestHelpersByName(name string) ([]model.Service, bool) {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	services, ok := s.availableTestHelpers[name]
 	return services, ok
 }
@@ -181,12 +292,7 @@ func (s *Session) MaybeLookupLocation() error {
 
 // MaybeLookupBackends is a caching OONI backends lookup call.
 func (s *Session) MaybeLookupBackends() error {
-	return s.maybeLookupBackends(context.Background())
-}
-
-// MaybeLookupBackendsContext is like MaybeLookupBackends but with context.
-func (s *Session) MaybeLookupBackendsContext(ctx context.Context) (err error) {
-	return s.maybeLookupBackends(ctx)
+	return s.MaybeLookupBackendsContext(context.Background())
 }
 
 // ErrAlreadyUsingProxy indicates that we cannot create a tunnel with
@@ -207,6 +313,7 @@ var ErrAlreadyUsingProxy = errors.New(
 //
 // The tunnel will be closed by session.Close().
 func (s *Session) MaybeStartTunnel(ctx context.Context, name string) error {
+	// TODO(bassosimone): see if we can unify tunnelMu and mu.
 	s.tunnelMu.Lock()
 	defer s.tunnelMu.Unlock()
 	if s.tunnel != nil && s.tunnelName == name {
@@ -252,11 +359,15 @@ func (s *Session) NewExperimentBuilder(name string) (*ExperimentBuilder, error)
 // OONI probe services. This function will benchmark the available
 // probe services, and select the fastest. In case all probe services
 // seem to be down, we try again applying circumvention tactics.
+// This function will fail IMMEDIATELY if given a cancelled context.
 func (s *Session) NewProbeServicesClient(ctx context.Context) (*probeservices.Client, error) {
-	if err := s.maybeLookupBackends(ctx); err != nil {
+	if ctx.Err() != nil {
+		return nil, ctx.Err() // helps with testing
+	}
+	if err := s.maybeLookupBackendsContext(ctx); err != nil {
 		return nil, err
 	}
-	if err := s.MaybeLookupLocationContext(ctx); err != nil {
+	if err := s.maybeLookupLocationContext(ctx); err != nil {
 		return nil, err
 	}
 	if s.selectedProbeServiceHook != nil {
@@ -307,6 +418,8 @@ func (s *Session) ProbeASNString() string {
 
 // ProbeASN returns the probe ASN as an integer.
 func (s *Session) ProbeASN() uint {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	asn := geolocate.DefaultProbeASN
 	if s.location != nil {
 		asn = s.location.ASN
@@ -316,6 +429,8 @@ func (s *Session) ProbeASN() uint {
 
 // ProbeCC returns the probe CC.
 func (s *Session) ProbeCC() string {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	cc := geolocate.DefaultProbeCC
 	if s.location != nil {
 		cc = s.location.CountryCode
@@ -325,6 +440,8 @@ func (s *Session) ProbeCC() string {
 
 // ProbeNetworkName returns the probe network name.
 func (s *Session) ProbeNetworkName() string {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	nn := geolocate.DefaultProbeNetworkName
 	if s.location != nil {
 		nn = s.location.NetworkName
@@ -334,6 +451,8 @@ func (s *Session) ProbeNetworkName() string {
 
 // ProbeIP returns the probe IP.
 func (s *Session) ProbeIP() string {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	ip := geolocate.DefaultProbeIP
 	if s.location != nil {
 		ip = s.location.ProbeIP
@@ -353,6 +472,8 @@ func (s *Session) ResolverASNString() string {
 
 // ResolverASN returns the resolver ASN
 func (s *Session) ResolverASN() uint {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	asn := geolocate.DefaultResolverASN
 	if s.location != nil {
 		asn = s.location.ResolverASN
@@ -362,6 +483,8 @@ func (s *Session) ResolverASN() uint {
 
 // ResolverIP returns the resolver IP
 func (s *Session) ResolverIP() string {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	ip := geolocate.DefaultResolverIP
 	if s.location != nil {
 		ip = s.location.ResolverIP
@@ -371,6 +494,8 @@ func (s *Session) ResolverIP() string {
 
 // ResolverNetworkName returns the resolver network name.
 func (s *Session) ResolverNetworkName() string {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	nn := geolocate.DefaultResolverNetworkName
 	if s.location != nil {
 		nn = s.location.ResolverNetworkName
@@ -412,17 +537,10 @@ func (s *Session) UserAgent() (useragent string) {
 	return
 }
 
-// MaybeUpdateResources updates the resources if needed.
-func (s *Session) MaybeUpdateResources(ctx context.Context) error {
-	return (&resources.Client{
-		HTTPClient: s.DefaultHTTPClient(),
-		Logger:     s.logger,
-		UserAgent:  s.UserAgent(),
-		WorkDir:    s.assetsDir,
-	}).Ensure(ctx)
-}
-
-func (s *Session) getAvailableProbeServices() []model.Service {
+// getAvailableProbeServicesUnlocked returns the available probe
+// services. This function WILL NOT acquire the mu mutex, therefore,
+// you MUST ensure you are using it from a locked context.
+func (s *Session) getAvailableProbeServicesUnlocked() []model.Service {
 	if len(s.availableProbeServices) > 0 {
 		return s.availableProbeServices
 	}
@@ -457,22 +575,27 @@ func (s *Session) initOrchestraClient(
 	return clnt, nil
 }
 
-// LookupASN maps an IP address to its ASN and network name. This method implements
-// LocationLookupASNLookupper.LookupASN.
-func (s *Session) LookupASN(dbPath, ip string) (uint, string, error) {
-	return geolocate.LookupASN(dbPath, ip)
-}
-
 // ErrAllProbeServicesFailed indicates all probe services failed.
 var ErrAllProbeServicesFailed = errors.New("all available probe services failed")
 
-func (s *Session) maybeLookupBackends(ctx context.Context) error {
-	// TODO(bassosimone): do we need a mutex here?
+// maybeLookupBackendsContext uses testMaybeLookupBackendsContext if
+// not nil, otherwise it calls MaybeLookupBackendsContext.
+func (s *Session) maybeLookupBackendsContext(ctx context.Context) error {
+	if s.testMaybeLookupBackendsContext != nil {
+		return s.testMaybeLookupBackendsContext(ctx)
+	}
+	return s.MaybeLookupBackendsContext(ctx)
+}
+
+// MaybeLookupBackendsContext is like MaybeLookupBackends but with context.
+func (s *Session) MaybeLookupBackendsContext(ctx context.Context) error {
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	if s.selectedProbeService != nil {
 		return nil
 	}
 	s.queryProbeServicesCount.Add(1)
-	candidates := probeservices.TryAll(ctx, s, s.getAvailableProbeServices())
+	candidates := probeservices.TryAll(ctx, s, s.getAvailableProbeServicesUnlocked())
 	selected := probeservices.SelectBest(candidates)
 	if selected == nil {
 		return ErrAllProbeServicesFailed
@@ -490,19 +613,33 @@ func (s *Session) LookupLocationContext(ctx context.Context) (*geolocate.Results
 	// when we are using a proxy because that might leak information.
 	task := geolocate.Must(geolocate.NewTask(geolocate.Config{
 		EnableResolverLookup: s.proxyURL == nil,
-		HTTPClient:           s.DefaultHTTPClient(),
 		Logger:               s.Logger(),
-		ResourcesManager:     s,
+		Resolver:             s.resolver,
 		UserAgent:            s.UserAgent(),
 	}))
 	return task.Run(ctx)
 }
 
+// lookupLocationContext calls testLookupLocationContext if set and
+// otherwise calls LookupLocationContext.
+func (s *Session) lookupLocationContext(ctx context.Context) (*geolocate.Results, error) {
+	if s.testLookupLocationContext != nil {
+		return s.testLookupLocationContext(ctx)
+	}
+	return s.LookupLocationContext(ctx)
+}
+
 // MaybeLookupLocationContext is like MaybeLookupLocation but with a context
-// that can be used to interrupt this long running operation.
+// that can be used to interrupt this long running operation. This function
+// will fail IMMEDIATELY if given a cancelled context.
 func (s *Session) MaybeLookupLocationContext(ctx context.Context) error {
+	if ctx.Err() != nil {
+		return ctx.Err() // helps with testing
+	}
+	defer s.mu.Unlock()
+	s.mu.Lock()
 	if s.location == nil {
-		location, err := s.LookupLocationContext(ctx)
+		location, err := s.lookupLocationContext(ctx)
 		if err != nil {
 			return err
 		}
